unit WharfModules;

// Version 1.3
// Added WharfModules to Groups
// Version 1.2
// Made UnLoadWharfModules recursive
// Version 1.l
// Fixed Crash-on-Recycle when a module wasn't loaded properly
// Version 1.0

interface
Uses
  BoxFiles,
  BoxAPI,
  MyTypes,
  Windows;

Procedure LoadWharfModules(Box: PBox);
Procedure UnLoadWharfModules(WharfModule: PWharfModule);

implementation

Function ModuleWndProc(Wnd, Msg: Integer; wParam, lParam: Longint): Longint; stdcall;
Begin
  Result := DefWindowProcA(Wnd,Msg,wParam,lParam);
End;

Procedure LoadWharfModules(Box: PBox);
Var
  Buffer: String;
  WharfModule: PWharfModule;
  wdata: PwharfDataType;
  wc: TWndClass;
  Flags: String;

Begin
  Buffer := BoxReadNextConfig(Box^.BoxPtr,'*WharfModule');
    While buffer<>'' do
      Begin
        New(WharfModule);

        WharfModule^.Next_Module := Box^.First_WharfModule;
        Box^.First_WharfModule := WharfModule;

        With WharfModule^ Do
        Begin
          ModuleHandle := LoadLibrary( PChar(GetToken(Buffer,5) ));

          If ModuleHandle = 0 then
            ModuleHandle := LoadLibrary( PChar(LsPath+'\'+GetToken(Buffer,5) ));

          If ModuleHandle <> 0 then
          Begin

            initModule := GetProcAddress( ModuleHandle, 'initWharfModule' );
            quitModule := GetProcAddress( ModuleHandle, 'quitWharfModule' );

            New(wdata);
            wdata^.lsPath := PChar( LsPath );
            wdata^.pixmapDir := StrAlloc( 255 );
            wdata^.pos := 0;
            wdata^.borderSize := 0;
            wdata^.defaultBmp := '';

            If NOT GetClassInfo( Inst, 'LsBoxModuleContainer', wc ) then
            Begin
              FillChar( wc,sizeof(wc), #0 );
              wc.lpfnWndProc := @ModuleWndProc;
              wc.lpszClassName := 'LsBoxModuleContainer';
              wc.hInstance := Inst;
              RegisterClassA( wc );
            End;

            xPos := Str2Int(GetToken(Buffer,1));
            yPos := Str2Int(GetToken(Buffer,2));
            Width := Str2Int(GetToken(Buffer,3));
            Height := Str2Int(GetToken(Buffer,4));

            M_Wnd := CreateWindowExA(
                       0,'LsBoxModuleContainer',nil,WS_CHILD,
                       xPos,yPos,Width,Height,Box^.hmainWnd,0,
                       Inst,nil );
            SetWindowPos( M_Wnd,HWND_TOP,xPos,yPos,
                            Width,Height,SWP_HIDEWINDOW );

            If (@initModule <> nil) then
              initModule( M_Wnd, Inst,wdata );

            StrDispose( wdata^.pixmapDir );
            Dispose( wdata );

            Flags := GetToken(Buffer,6);

            If (Flags<>'') then
              GroupItem := AddGroupItem(M_Wnd,GetGroup(Flags))
            Else
              GroupItem := AddGroupItem(M_Wnd,0);

            If (Boolean(CheckFlag(Flags,'H'))or(GetFlag(GroupItem^.Group^.Flags,Hidden))) Then
              ShowWindow(M_Wnd,SW_Hide)
            else
              ShowWindow(M_Wnd,SW_Show);
          End
          Else
            MsgBox(MB_ICONWARNING,'Can''t load WharfModule:'+#13+GetToken(Buffer,1));

        End;

        Buffer := BoxReadNextConfig(Box^.BoxPtr,'*WharfModule');
      End;
End;

Procedure UnLoadWharfModules(WharfModule: PWharfModule);
Begin
 If ( WharfModule<>nil ) then
  With WharfModule^ Do
  Begin
    If ModuleHandle <> 0 then
      Begin
        If (@quitModule <> nil) then
          quitModule(Inst);
        FreeLibrary(ModuleHandle);
      End;

      if (GroupItem^.Prev_Item = nil) then
      begin
        GroupItem^.Group^.First_Item := GroupItem^.Next_Item;
        if (GroupItem^.Next_Item <> nil) then
          GroupItem^.Next_Item^.Prev_Item := nil;
      end else
        if (GroupItem^.Next_Item = nil) then
          GroupItem^.Prev_Item^.Next_Item := nil
        else begin
          GroupItem^.Next_Item^.Prev_Item := GroupItem^.Prev_Item;
          GroupItem^.Prev_Item^.Next_Item := GroupItem^.Next_Item;
        end;
      Dispose(GroupItem);
      UnLoadWharfModules(Next_Module);
      DestroyWindow(M_Wnd);
      Dispose(WharfModule);
  End;
End;

end.
